define(["jquery", "underscore", "grid"], function ($, _, Grid) {
  var DateTimeEditor = Editor.extend({
    types: [],
    name: "DateTimeEditor",
    datepicker: null,
    format: 'ddd mmm dd yyyy',
    remainder: null,
    originalDate: null,
    render: function () {
      if (!this.editor) {

        // create editor elements
        this.editor = document.createElement("div");
        this.editor.className = "sensei-grid-editor sensei-grid-date-editor";
        var $wrapper = $("<div>", {class: "sensei-grid-date-wrapper"});
        $wrapper.append($("<input>", {type: "text", class: "datepicker"}));
        $(this.editor).append($wrapper);
        this.grid.$el.append(this.editor);

        var format = this.grid.config.dateFormat || this.format;

        var editor = this;

        $("input", editor.editor).on("change", function (e) {
          if (!e.isTrigger) {
            //console.log("[TEST] input manually changed", $(this).val(), e);
            editor.originalDate = $(this).val();
          }
        });

        // load the datepicker
        $('.datepicker').pickadate({
          format: format,
          editable: true,
          today: false,
          clear: false,
          close: false,
          onSet: function (context) {
            // if current event is date select
            if (context.select) {

              // get formatted date from date picker
              var date = this.get("select", "yyyy-mm-dd");

              // get original date
              var originalDate = editor.originalDate;

              if (!originalDate) {
                return;
              }

              // get remainder from original date
              var remainder = originalDate.split(" ");
              remainder.shift();

              // check if there is remainder that needs to be added to date
              if (remainder && _.isArray(remainder)) {
                // prepend remainder array with date
                remainder.unshift(date);
                // create full datetime string by joining parts together
                date = remainder.join(" ");
              }

              // set value in input
              $("input", editor.editor).val(date).focus();
            }
          }
        });

        // store datepicker instance
        this.datepicker = $(".datepicker").pickadate('picker');
      }
    },
    show: function () {
      this.getElement().show();
      // force open datepicker
      if (this.datepicker) {
        this.datepicker.open();
      }
    },
    getValue: function () {
      return $("input", this.editor).val();
    },
    setValue: function (val) {
      // set value in input of this editor and focus it
      $("input", this.editor).val(val).focus();

      // store original date
      this.originalDate = val;

      // if datepicker is active, highlight current value as date
      if (this.datepicker) {
        // highlight current date
        this.datepicker.set('highlight', val);
      }
    }
  });
  return DateTimeEditor;
});