define(['underscore', 'backbone', 'helpers/string', 'context', 'cookiesjs'],
    function (_, Backbone, string, Context, Cookies) {
        var License = Backbone.Model.extend({
            defaults: {
                license: null,
                expires_at: false
            },
            types: {
                1: "Personal",
                2: "Commercial",
                3: "Beta"
            },
            parse: function (item) {
                var current_user_count = item["current_user_count"];

                // flatten root object
                if (_.isObject(item["license"])) {
                    item = item["license"];
                }
                //item["is_active"] = item["is_active"] ? "Active" : "Inactive";
                item["license_type"] = this.types[item["license_type"]];
                item["current_user_count"] = current_user_count;
                return item;
            },
            urlRoot: "/user/license",
            getErrorMessage: function () {

              // check if expires_at is set
              if (this.get("is_active") !== true) {
                return "License key not found, please add a license to activate Datazenit.";
              }

              // check user count
              if (parseInt(this.get("current_user_count")) > parseInt(this.get("user_count"))) {
                return "User limit for currently active license exceeded. Please upgrade your license."
              }

              // check expiration date if not empty
              // this is only needed for beta/trial licenses
              if (!_.isEmpty(this.get("expires_at")) &&
                  new Date(this.get("expires_at")) < new Date()) {
                return "Your license has expired, please add a new one.";
              }

              return null;
            },
            isValid: function () {
              return this.getErrorMessage() === null;
            }
        });
        License.hasValidLicense = function () {
            var expires_at = Cookies.get("license_expires_at");

            if (!expires_at) {
                return false;
            }

            return new Date(expires_at) > new Date();
        };
        License.setLicense = function (license) {
            Cookies.set("license_expires_at", license.get("expires_at") || "");
        };
        License.clearLicense = function () {
            // remove expires at cookie
            Cookies.expire("license_expires_at");
        };
        return License;
    });
