define(['underscore', 'backbone', 'context'], function (_, Backbone, Context) {
  var Query = Backbone.Model.extend({
    base_url: "/query/%(connection)s?db=%(db)s",
    base_insert_url: "/query/insert/%(connection)s?db=%(db)s",
    defaults: {
      meta: {},
      q: null,
      rows: []
    },
    url: function () {
      return this.base_url.format(Context);
    },
    insertURL: function () {
      return this.base_insert_url.format(Context);
    },
    /**
     * Replace order by statement in a query
     * @param newOrderBy Order by inner statement, e.g., "id desc"
     */
    replaceOrderBy: function (newOrderBy) {
      var matchOrderBy = /(ORDER\s+BY\s+)([\s\S]+?)(\s+FOR\s+|\s+OFFSET|\s+LOCK\s+IN|\s+PROCEDURE|\s+LIMIT|\s+INTO\s+OUTFILE|;|$)/i;
      var addOrderBy = /(\s+FOR\s+|\s+OFFSET|\s+LOCK\s+IN|\s+PROCEDURE|\s+LIMIT|\s+INTO\s+OUTFILE|;|$)/i;
      var isSelect = /^\s*SELECT\s+/i;

      var query = this.get("q");

      // check if query is not empty and is string
      if (!_.isString(query) || _.isEmpty(query)) {
        return null;
      }

      // check if query is select
      if (!query.match(isSelect)) {
        return null;
      }

      // check if there is order by statement in query
      if (query.match(matchOrderBy)) {
        // replace current order by statement with a new one
        return query.replace(matchOrderBy, "$1"+newOrderBy+"$3");
      } else {
        // existing order by not found, add order by statement to query
        return query.replace(addOrderBy, " ORDER BY "+newOrderBy+"$1")
      }
    }
  });
  return Query;
});