define(['context', 'models/query', 'helpers/string'], function (Context, Query) {
  describe("Query", function () {

    var newOrderBy = "foo ASC";
    var q = new Query();

    it("should return correct urls", function () {

      // mock context
      Context.connection = 1;
      Context.db = "test";
      Context.table = "test_table";
      Context.driver = "postgresql";

      expect(q.url()).toBe("/query/1?db=test");
      expect(q.insertURL()).toBe("/query/insert/1?db=test");
    });

    describe("replaceOrderBy", function () {
      it("should replace ORDER BY statement", function () {

        q.set("q", "SELECT * FROM test_table ORDER BY id DESC");
        expect(q.replaceOrderBy(newOrderBy)).toBe("SELECT * FROM test_table ORDER BY foo ASC");

        q.set("q", "SELECT * FROM test_table ORDER BY id DESC;");
        expect(q.replaceOrderBy(newOrderBy)).toBe("SELECT * FROM test_table ORDER BY foo ASC;");

        q.set("q", "  SELECT * FROM test_table ORDER BY id DESC  ");
        expect(q.replaceOrderBy(newOrderBy)).toBe("  SELECT * FROM test_table ORDER BY foo ASC");

        q.set("q", "\n\n   SELECT \n\n   * FROM test_table \n\n  ORDER BY \n\n id DESC\n\n;");
        expect(q.replaceOrderBy(newOrderBy)).toBe("\n\n   SELECT \n\n   * FROM test_table \n\n  ORDER BY \n\n foo ASC;");

        q.set("q", "SELECT * FROM test_table ORDER BY id DESC LIMIT 100");
        expect(q.replaceOrderBy(newOrderBy)).toBe("SELECT * FROM test_table ORDER BY foo ASC LIMIT 100");

        q.set("q", "SELECT * FROM test_table ORDER BY id desc into outfile");
        expect(q.replaceOrderBy(newOrderBy)).toBe("SELECT * FROM test_table ORDER BY foo ASC into outfile");

        q.set("q", "SELECT * FROM test_table   ORDER  BY  id   DESC  ");
        expect(q.replaceOrderBy(newOrderBy)).toBe("SELECT * FROM test_table   ORDER  BY  foo ASC");

        q.set("q", "SELECT * FROM test_table ORDER BY rand();");
        expect(q.replaceOrderBy(newOrderBy)).toBe("SELECT * FROM test_table ORDER BY foo ASC;");
      });

      it("should add ORDER BY statement", function () {

        q.set("q", "SELECT * FROM test_table");
        expect(q.replaceOrderBy(newOrderBy)).toBe("SELECT * FROM test_table ORDER BY foo ASC");

        q.set("q", "SELECT * FROM test_table;");
        expect(q.replaceOrderBy(newOrderBy)).toBe("SELECT * FROM test_table ORDER BY foo ASC;");

        q.set("q", "SELECT * FROM test_table LIMIT 0, 10");
        expect(q.replaceOrderBy(newOrderBy)).toBe("SELECT * FROM test_table ORDER BY foo ASC LIMIT 0, 10");
      });

      it("should return null when query can't be parsed", function () {

        q.set("q", "");
        expect(q.replaceOrderBy(newOrderBy)).toBeNull();

        q.set("q", null);
        expect(q.replaceOrderBy(newOrderBy)).toBeNull();

        q.set("q", undefined);
        expect(q.replaceOrderBy(newOrderBy)).toBeNull();

        q.set("q", 0);
        expect(q.replaceOrderBy(newOrderBy)).toBeNull();

        q.set("q", "    ");
        expect(q.replaceOrderBy(newOrderBy)).toBeNull();

        q.set("q", "INSERT INTO test_table (1,2,3);");
        expect(q.replaceOrderBy(newOrderBy)).toBeNull();

        q.set("q", "SSELEECT FROM ORDER BY id DESC LIMIT 1;");
        expect(q.replaceOrderBy(newOrderBy)).toBeNull();

        q.set("q", "FOO BAR SELECT * FROM ORDER BY id DESC LIMIT 1");
        expect(q.replaceOrderBy(newOrderBy)).toBeNull();
      });

    });
  });
});