define(['context', 'views/connections', 'models/connection', 'collections/connections', 'bootstrap', 'jquery'],
    function (Context, ConnectionsView, Connection, Connections, Bootstrap, $) {
    describe("ConnectionsView", function () {

        // set context
        Context.db = "test_db";
        Context.section = "main";
        Context.connection = 1;

        // disable ajax
        Backbone.ajax = function () {};

        var c1 = new Connection({id: 1, title: "Test Connection", port: 3306, username: "foo", hostname: "localhost"});
        var c2 = new Connection({id: 2, title: "Test Connection 2", port: 3306, username: "bar", hostname: "remote"});
        var view = new ConnectionsView({active: c1.id});

        var r = view.render();
        view.connections.add(c1);
        view.connections.add(c2);

        it("should render logo", function () {
            expect($(r.el).find(".logo-wrapper").length).toEqual(1);
            expect($(r.el).find(".logo-wrapper img").length).toEqual(1);
        });
        it("should render logout element", function () {
            expect($(r.el).find('a[href="/logout"]').length).toEqual(1);
        });
        it("should render connections", function () {
            expect($(r.el).find(".connection-item").length).toEqual(2);
        });
        it("should set connection titles", function () {
            expect($(r.el).find(".connection:first").text()).toEqual("Test Connection 2");
            expect($(r.el).find(".connection:last").text()).toEqual("Test Connection");
        });
        it("should set connection permalinks", function () {
            //expect($(r.el).find("li:not(.action-tab):first a").attr("href")).toEqual("#/content/main/1");
            expect($(r.el).find("li:not(.action-tab):last a").attr("href")).toEqual("/logout");
        });
        it("should set active connection", function () {
            expect($(r.el).find("li.active").length).toEqual(1);

            var $first = $(r.el).find("li:not(.action-tab):first");
            expect($first.hasClass("active")).toBeFalsy();
        });
        it("classNameCbk should return active class name for a given connection", function () {
            expect(view.classNameCbk(c1)).toEqual("connection-item active");
            expect(view.classNameCbk(c2)).toBe("connection-item");
        });
        it("clicking on connection should set it active", function () {
            $(r.el).find("li:not(.action-tab):first .connection").click();
            expect($(r.el).find("li:not(.action-tab):last").hasClass("active")).toBeFalsy();
            expect($(r.el).find("li:not(.action-tab):first").hasClass("active")).toBeTruthy();
        });
        it("new connection link should remove active class from other connections", function () {
            $(r.el).find(".create-new-connection").click();
            expect($(r.el).find("li:not(.action-tab)").hasClass("active")).toBeFalsy();
        });
    });
});