define(['jquery', 'underscore', 'backbone', 'tpls', 'events', 'require', 'views/content/query/filter', "context",
        "cm/lib/codemirror", 'views/content/query/history/list', "baseview", "models/query",
        "cm/mode/sql/sql", "cm/addon/hint/show-hint", "cm/addon/hint/sql-hint"],
    function ($, _, Backbone, Templates, Events, require, FilterView, Context,
      CodeMirror, HistoryList, BaseView, Query) {
        var RawView = BaseView.extend({
            template: Templates["content/query/raw"],
            events: {
                "click .execute-query": "executeQuery"
            },
            defaultQuery: "SELECT 1 as test;",
            orderQuery: null,
            initialize: function () {

                // enable output panel, because it is needed for this view
                Context.output = true;

                _.bindAll(this, "resetQuery", "executeQuery", "clearOrderQuery", "tableShortcut");

                // query must be persistent, so don't change query on db/table change
                //this.listenTo(Events, "table:change", this.setTable);
                //this.listenTo(Events, "db:change", this.resetQuery);

                this.listenTo(Events, "table:shortcut:post", this.tableShortcut);
                this.listenTo(Events, "query:set", this.setQuery);
                this.listenTo(Events, "query:load", this.setQuery);
                this.listenTo(Events, "data:sort", this.sortData, this);
                this.listenTo(Events, "table:change", this.clearOrderQuery, this);
                //this.listenTo(Events, "table:change", this.focusEditor, this);
                this.listenTo(Events, "tables:fetched table:change", this.updateHints, this);
            },
            clearOrderQuery: function () {
              this.orderQuery = null;
            },
            setQuery: function (query) {
                if (query instanceof Backbone.Model) {
                  query = query.get("query");
                }
                Context.setLastRawQuery(query);
                this.cm.setValue(query);
            },
            tableShortcut: function (table) {
              var query = Context.getDefaultQuery(table.get("title"));
              this.cm.setValue(query.toString());
              this.cm.focus();

              this.executeQuery();
            },
            sortData: function (col, order) {

              // trim query from spaces and semi-colons
              var query = _.trim(Context.query, "; ");

              // create new query model instance
              var queryModel = new Query();
              queryModel.set("q", query);

              // set new order statement
              var newOrderBy = col + " " + order;

              // generate a new query and save as orderQuery
              this.orderQuery = queryModel.replaceOrderBy(newOrderBy);

              // execute query, can't use executeQuery here, because it must reset orderQuery
              //this.executeQuery();
              Events.trigger("query:execute");
            },
            // @deprecated
            setTable: function (table) {
                // build default query with squel
                //var view = new FilterView();
                //var query = view.buildBaseQuery(table.get("title"));
                // @FIXME can't call filter view because of duplicate event listeners
                // two options here
                // * 1) manually remove the fv view after getting needed info
                // 2) refactor query generation outside of fv
                //var query = "SELECT * FROM " + table.get("title");
                var fv = new FilterView();
                var query = fv.buildBaseQuery(table.get("title"));
                fv.remove();
                this.cm.setValue(query.toString());
                this.cm.focus();
            },
            executeQuery: function () {
                // reset order query
                this.orderQuery = null;

                // store current query in context for later use
                //Context.last_raw_query = this.getQuery();
                Context.setLastRawQuery(this.getQuery());

                // trigger query execute event
                Events.trigger("query:execute");
            },
            runSelection: function () {
                return this.$(".run-selection").is(":checked");
            },
            getQuery: function () {

                // check if order query exists
                if (this.orderQuery) {
                  // return orderQuery
                  return this.orderQuery;
                }

                if (!_.isObject(this.cm)) {
                    return null;
                }

                // check if there is an active selection and user has
                // selected the option to run selections individually
                if (this.runSelection() && this.cm.somethingSelected()) {
                    // return only selected part
                    return this.cm.getSelection();
                }

                // no selection, return full content
                return this.cm.getValue();
            },
            resetQuery: function () {

                // do nothing here, query must stay persistent

                // reset last raw query
                //Context.last_raw_query = null;

                //if (Context.table === null) {
                //    // set default query in editor
                //    this.cm.setValue(this.defaultQuery);
                //}
            },
            focusEditor: function () {
                this.cm.focus();
            },
            updateHints: function () {
              // create table object
              var tables = _.object(Context.tables, []);
              tables[Context.table] = Context.meta["columns"];

              // update table hinting for query editor
              this.cm.setOption("hintOptions", {tables: tables});
            },
            saveCurrentQuery: function (cm) {
              Context.setLastRawQuery(cm.getValue());
            },
            render: function () {

                this.$el.html(this.template());

                // hints for current table only, because all tables are not yet loaded
                var tables = _.object(Context.tables, []);
                tables[Context.table] = Context.meta["columns"];

                var last = Context.getLastRawQuery();
                this.cm = CodeMirror(this.$(".editor").get(0), {
                    lineNumbers: true,
                    mode: "text/x-mysql",
                    // important to pass string, causes bugs that are hard to debug
                    value: last || "",
                    extraKeys: {
                        "Ctrl-Space": "autocomplete",
                        "Ctrl-Enter": this.executeQuery
                    },
                    hintOptions: {
                        tables: tables
                    }
                    //viewportMargin: Infinity
                });

                // save current query on editor change
                this.cm.on("changes", this.saveCurrentQuery);

                // @FIXME proper on load or use refresh() after setValue
                var that = this;
                setTimeout(function () {

                    //if (that.getQuery().trim().length === 0) {
                    //    that.resetQuery();
                    //}

                    // check if we can restore last raw query
                    //if (Context.getLastRawQuery()) {
                    //  // restore last raw query
                    //  that.cm.setValue(Context.getLastRawQuery());
                    //}

                    // refresh codemirror element
                    that.cm.refresh();

                    // focus raw query editor
                    that.cm.focus();
                }, 100);

                // create history list view
                this.subviews.hv = new HistoryList({parent: this});
                this.$(".extra-wrapper").html(this.subviews.hv.render().el);

                return this;
            }
        });
        return RawView;
    });
