define(["jquery", "underscore", "backbone", "tpls", "events", "context", "baseview", "collections/users",
      "views/user/team_member", "models/invite"],
    function ($, _, Backbone, Templates, Events, Context, BaseView, Users, TeamMember, Invite) {
      var TeamView = BaseView.extend({
        template: Templates["user/team"],
        initialize: function () {
          _.bindAll(this, "refresh");
          this.users = new Users();
          this.listenTo(this.users, "add", this.addUser, this);
        },
        events: {
          "submit .invite-team-member": "inviteTeamMember",
          "click .refresh": "refresh"
        },
        inviteTeamMember: function (e) {
          if (e) { e.preventDefault(); }
          // remove old error messages
          this.$("p.error").remove();

          // main form
          var $form = this.$(".invite-team-member");

          // save button
          var $btn = this.$(".save-form");
          $btn.button('loading');

          // success message element
          var $success = this.$(".alert-success");
          $success.hide();

          // email input
          var $input = this.$("input[name=receiver_email]");

          // serialize data from form
          var data = Context.serializeObj($(e.currentTarget));

          // create new model and save it
          var invite = new Invite();
          invite.save(data, {
            success: function (model, res) {
              $input.val("");
              $success.fadeIn();
              var invite_url = res.invite_url;
              $success.find(".invite_url").attr("href", invite_url).text(invite_url);
            },
            error: function (model, xhr) {
              var unexpected = "Failed to send invite, please try again later or contact support.";
              if (xhr.responseJSON && xhr.responseJSON.errors && _.isObject(xhr.responseJSON.errors)) {
                _.each(xhr.responseJSON.errors, function (error) {
                  var $error = $("<p>").addClass("error help-block").text(error || unexpected);
                  $form.append($error);
                });
              } else {
                alert(unexpected);
              }
            }
          }).always(function () {
            $btn.button("reset");
          });
        },
        renderWithUser: function () {
          if (!Context.user) {
            alert("Current user not found, please logout and log in back again.");
            return;
          }

          var signup_approval = Context.connections && Context.connections.signup_approval ?
              Context.connections.signup_approval : false;

          //var max_user_count = Context.getMaxUserCount();

          // render template
          this.$el.html(this.template({
            user: Context.user.toJSON(),
            signup_approval: signup_approval
          }));

          // get user list
          this.$(".loading-row").show();
          this.$(".empty-row").hide();
          this.users.reset();
          this.users.fetch();
        },
        refresh: function (e) {
          e.preventDefault();
          this.render();
        },
        render: function () {
          if (!Context.user) {
            this.listenTo(Events, "connections:fetched", this.renderWithUser);
          } else {
            this.renderWithUser();
          }

          return this;
        },
        addUser: function (model) {
          if (this.$(".loading-row").is(":visible")) {
            this.$(".loading-row").hide();
          }
          this.subviews[model.cid] = new TeamMember(model);
          this.$("table>tbody").append(this.subviews[model.cid].render().el);
        }
      });
      return TeamView;
    });
